﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="FileEditRequestEventArgs.cs" company="Sander van Vliet">
//   2011 Sander van Vliet
// </copyright>
// <summary>
//   The file edit request event args.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace TFDP.Common
{
    using System;

    /// <summary>
    /// The file edit request event args.
    /// </summary>
    public sealed class FileEditRequestEventArgs : EventArgs
    {
        #region Constants and Fields

        /// <summary>
        /// The file name.
        /// </summary>
        private string fileName;

        /// <summary>
        /// The result.
        /// </summary>
        private FileEditRequestResult result;

        #endregion

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="FileEditRequestEventArgs"/> class.
        /// </summary>
        /// <param name="fileName">
        /// The file name.
        /// </param>
        public FileEditRequestEventArgs(string fileName)
        {
            this.fileName = fileName;
            this.result = FileEditRequestResult.Empty;
        }

        #endregion

        #region Public Properties

        /// <summary>
        /// Gets FileName.
        /// </summary>
        public string FileName
        {
            get
            {
                return this.fileName;
            }
        }

        /// <summary>
        /// Gets Result.
        /// </summary>
        public FileEditRequestResult Result
        {
            get
            {
                return this.result;
            }
        }

        #endregion
    }
}